<?php
/*
Plugin Name: Mx Smart Plugin Manager (Multisite Ready)
Plugin URI: https://www.wp-diary.com
Description: 多站点兼容的插件管理器，支持每个用户独立切换后台模式，以及每个子站单独插件过滤。
Version: 2.2
Author: WordPress日记
*/

if (!defined('ABSPATH')) exit;

// 启动 session，并确保每个站点独立
add_action('init', function () {
    if (!is_admin() || wp_doing_ajax() || defined('DOING_CRON') || defined('REST_REQUEST')) return;
    if (session_status() !== PHP_SESSION_ACTIVE) {
        session_start();
    }
});

// 添加操作按钮
function mx_add_simplified_mode_button($actions, $plugin_file, $plugin_data, $context) {
    if (current_user_can('administrator') && !is_network_admin()) {
        $blog_id = get_current_blog_id();
        $session_key = "MiniMode_{$blog_id}";
        $mode = $_SESSION[$session_key] ?? 'full';
        $disabled_plugins = get_option("_simplified_disabled_plugins_{$blog_id}", []);
        if($mode == 'simple'){
            $actions = [];
        } else {
            if (in_array($plugin_file, $disabled_plugins)) {
                $actions['simplified_disable'] = '<a style="color:red" href="' . wp_nonce_url(admin_url('admin-post.php?action=toggle_simplified_mode_plugin&plugin=' . urlencode($plugin_file)), 'toggle_simplified_mode_plugin') . '">' . __('在简化模式中启用', 'mx-smart-plugin-manager') . '</a>';
            } else {
                $actions['simplified_disable'] = '<a style="color:green" href="' . wp_nonce_url(admin_url('admin-post.php?action=toggle_simplified_mode_plugin&plugin=' . urlencode($plugin_file)), 'toggle_simplified_mode_plugin') . '">' . __('在简化模式中禁用', 'mx-smart-plugin-manager') . '</a>';
            }
        }
    }
    return $actions;
}
add_filter('plugin_action_links', 'mx_add_simplified_mode_button', 10, 4);

// 启用/禁用插件处理
function mx_toggle_simplified_mode_plugin() {
    if (!isset($_GET['plugin']) || !check_admin_referer('toggle_simplified_mode_plugin')) {
        wp_die('非法请求');
    }

    if (is_network_admin()) {
        wp_die('请在子站后台操作。');
    }

    $plugin_file = sanitize_text_field($_GET['plugin']);
    $blog_id = get_current_blog_id();

    $option_name = "_simplified_disabled_plugins_{$blog_id}";
    $disabled_plugins = get_option($option_name, []);

    if (in_array($plugin_file, $disabled_plugins)) {
        $disabled_plugins = array_diff($disabled_plugins, [$plugin_file]);
    } else {
        $disabled_plugins[] = $plugin_file;
    }

    update_option($option_name, array_values($disabled_plugins));
    wp_safe_redirect(admin_url('plugins.php'));
    exit;
}
add_action('admin_post_toggle_simplified_mode_plugin', 'mx_toggle_simplified_mode_plugin');

// 管理工具栏模式切换按钮
function mx_admin_bar_mode_switch($wp_admin_bar) {
    if (is_admin() && !is_network_admin() && current_user_can('administrator')) {
        $blog_id = get_current_blog_id();
        $session_key = "MiniMode_{$blog_id}";

        // 检查 cookie 中的模式设置
        if(!isset($_SESSION[$session_key])){
            if(isset($_COOKIE[$session_key])){
                $cookie_mode = $_COOKIE[$session_key];
                $_SESSION[$session_key] = $cookie_mode;
                setcookie($session_key, $cookie_mode, time() + 3600 * 24 * 30, COOKIEPATH, COOKIE_DOMAIN, is_ssl(), true);
            }
        }
        $mode = $_SESSION[$session_key] ?? 'full';
        $mode_label = ($mode === 'simple') ? __('当前：简化模式', 'mx-smart-plugin-manager') : __('当前：完整模式', 'mx-smart-plugin-manager');
        $mode_color = ($mode === 'simple') ? 'background-color: green; padding: 3px 6px; color: #fff;' : 'background-color: red; padding: 3px 6px; color: #fff;';

        $wp_admin_bar->add_node([
            'id'    => 'plugin_mode_switch',
            'title' => "<span style='$mode_color'>$mode_label</span>",
            'href'  => wp_nonce_url(admin_url('admin-post.php?action=switch_mode'), 'switch_mode'),
        ]);
    }
}
add_action('admin_bar_menu', 'mx_admin_bar_mode_switch', 999);

// 模式切换逻辑
function mx_switch_mode() {
    if (!current_user_can('administrator') || !check_admin_referer('switch_mode')) {
        wp_die('非法请求');
    }

    if (is_network_admin()) {
        wp_die('请在子站后台操作。');
    }

    $blog_id = get_current_blog_id();
    $session_key = "MiniMode_{$blog_id}";
    $mode = $_SESSION[$session_key] ?? 'full';
    $new_mode = ($mode === 'full') ? 'simple' : 'full';

    // 设置 session 和 cookie
    $_SESSION[$session_key] = $new_mode;
    setcookie($session_key, $new_mode, time() + 3600 * 24 * 30, COOKIEPATH, COOKIE_DOMAIN, is_ssl(), true);

    wp_safe_redirect(wp_get_referer() ?: admin_url());
    exit;
}
add_action('admin_post_switch_mode', 'mx_switch_mode');

// 过滤插件列表
function mx_filter_active_plugins_in_simplified_mode($pre_option, $option, $default_value) {
    if ($option !== 'active_plugins') return false;
    if (!is_admin() || (defined('DOING_AJAX') && DOING_AJAX) || is_network_admin()) return false;

    $blog_id = get_current_blog_id();
    $session_key = "MiniMode_{$blog_id}";

    // 检查 cookie 中的模式设置
    if(!isset($_SESSION[$session_key])){
        if(isset($_COOKIE[$session_key])){
            $cookie_mode = $_COOKIE[$session_key];
            $_SESSION[$session_key] = $cookie_mode;
            setcookie($session_key, $cookie_mode, time() + 3600 * 24 * 30, COOKIEPATH, COOKIE_DOMAIN, is_ssl(), true);
        }
    }
    $mode = $_SESSION[$session_key] ?? 'full';

    if ($mode !== 'simple') {
        return false;
    }

    static $doing = false;
    if ($doing) return false;
    $doing = true;

    remove_filter('pre_option_active_plugins', 'mx_filter_active_plugins_in_simplified_mode', 10);
    $all_plugins = get_option('active_plugins', []);
    add_filter('pre_option_active_plugins', 'mx_filter_active_plugins_in_simplified_mode', 10, 3);

    $disabled_plugins = get_option("_simplified_disabled_plugins_{$blog_id}", []);
    $filtered_plugins = array_diff($all_plugins, $disabled_plugins);

    $doing = false;

    return array_values($filtered_plugins);
}
add_filter('pre_option_active_plugins', 'mx_filter_active_plugins_in_simplified_mode', 10, 3);
